// Copyright 2019, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Authors: Patrick Brosi <brosi@informatik.uni-freiburg.de>

#ifndef OSMFIXER_SERVER_STATSERVER_H_
#define OSMFIXER_SERVER_STATSERVER_H_

#include <map>
#include <string>
#include "osmfixer/index/StatIdx.h"
#include "util/http/Server.h"

namespace osmfixer {

typedef std::map<std::string, std::string> Params;

class StatServer : public util::http::Handler {
 public:
  explicit StatServer(const osmfixer::StatIdx* idx) : _idx(idx) {}

  virtual util::http::Answer handle(const util::http::Req& request,
                                    int connection) const;

 private:
  static std::string parseUrl(std::string u, std::string pl, Params* params);

  util::http::Answer handleMapReq(const Params& pars) const;
  util::http::Answer handleHeatMapReq(const Params& pars) const;
  util::http::Answer handleStatReq(const Params& pars) const;

  void printStation(const Station* stat, std::ostream* out) const;
  static void printGroup(const Group* stat, std::ostream* out);
  static void printSugg(const Suggestion* stat, std::ostream* out);

  const osmfixer::StatIdx* _idx;
};
}  // namespace osmfixer

#endif  // OSMFIXER_SERVER_STATSERVER_H_
